import unittest
from unittest.mock import patch
from unittest.mock import MagicMock
import re


class TestGetHelpMessage(unittest.TestCase):

    def test_help_message_wrapper_str(self):
        with patch('charlesbot.plugins.help_plugin.Help.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.help_plugin import Help
            mock_init.return_value = None
            slack_client = MagicMock()
            t_hlp = Help(slack_client)
            t_hlp.sc = slack_client
            help_str = t_hlp.get_help_message()
            regex = r'^```\n.*\n```$'
            p = re.compile(regex, re.MULTILINE | re.DOTALL)
            m = p.match(help_str)
            self.assertTrue(m)



import asynctest
from asynctest.mock import call
from asynctest.mock import patch
import json


class TestSendBroadcastMessage(asynctest.TestCase):

    def test_send_broadcast_message(self):
        with patch('charlesbot.plugins.broadcast_message.BroadcastMessage.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.broadcast_message import BroadcastMessage
            mock_init.return_value = None
            slack_client = asynctest.MagicMock()
            with patch('charlesbot.plugins.broadcast_message.slack_rtm_api_call') as mock_slack:  # NOQA
                user = {"real_name": "bob", "thumb_24": "tumb"}
                attachment = [
                    {
                        "fallback": "Broadcast message from bob - message",
                        "author_name": "bob",
                        "author_icon": "tumb",
                        "text": "message",
                    }
                ]
                t_bm = BroadcastMessage(slack_client)
                t_bm.sc = slack_client

                # No rooms
                t_bm.room_membership = {}
                yield from t_bm.send_broadcast_message("message", user)
                self.assertEqual(mock_slack.call_args, None)
                mock_slack.reset_mock()

                # One room
                t_bm.room_membership = {"1234": "fun"}
                yield from t_bm.send_broadcast_message("message", user)
                expected_call = call(
                    slack_client,
                    'chat.postMessage',
                    channel="1234",
                    attachments=json.dumps(attachment),
                    as_user=False,
                    username="Broadcast Message",
                    icon_url="https://s3-us-west-2.amazonaws.com/slack-files2/bot_icons/2015-07-26/8217958308_48.png"  # NOQA
                )
                self.assertEqual(len(mock_slack.call_args_list), 1)
                self.assertEqual(mock_slack.call_args_list[0], expected_call)
                mock_slack.reset_mock()

                # Two rooms
                t_bm.room_membership = {"1234": "fun", "4567": "funner"}
                yield from t_bm.send_broadcast_message("message", user)
                expected_call_1 = call(
                    slack_client,
                    'chat.postMessage',
                    channel="1234",
                    attachments=json.dumps(attachment),
                    as_user=False,
                    username="Broadcast Message",
                    icon_url="https://s3-us-west-2.amazonaws.com/slack-files2/bot_icons/2015-07-26/8217958308_48.png"  # NOQA
                )
                expected_call_2 = call(
                    slack_client,
                    'chat.postMessage',
                    channel="4567",
                    attachments=json.dumps(attachment),
                    as_user=False,
                    username="Broadcast Message",
                    icon_url="https://s3-us-west-2.amazonaws.com/slack-files2/bot_icons/2015-07-26/8217958308_48.png"  # NOQA
                )
                self.assertEqual(len(mock_slack.call_args_list), 2)
                self.assertTrue(expected_call_1 in mock_slack.call_args_list)
                self.assertTrue(expected_call_2 in mock_slack.call_args_list)
                mock_slack.reset_mock()



from charlesbot.util.parse import does_msg_contain_prefix
from charlesbot.util.parse import filter_message_types
import logging
import asyncio


class Help(object):

    def __init__(self, slack_client):
        self.is_running = True
        self._plugin_name = "Help!"
        self.log = logging.getLogger(__name__)
        self.sc = slack_client
        self.log.info("Initializing the Help! plugin")
        loop = asyncio.get_event_loop()
        self.q = asyncio.Queue()
        loop.create_task(self.consume())

    def get_plugin_name(self):
        return self._plugin_name

    def get_help_message(self):
        msg = [
            "!help - This help message",
            "!wall <msg> - Broadcast a message to all channels I'm a part of",
        ]
        help_msg = "\n".join(msg)
        return "```\n%s\n```" % help_msg

    @asyncio.coroutine
    def process_message(self, messages):
        for msg in messages:
            yield from self.parse_help_message(msg)

    @asyncio.coroutine
    def consume(self):
        while self.is_running:
            value = yield from self.q.get()
            loop = asyncio.get_event_loop()
            loop.create_task(self.process_message(value))

    @asyncio.coroutine
    def parse_help_message(self, msg):
        types = ['message']
        fields = ['channel', 'user', 'text']
        if not filter_message_types(msg, types, fields):
            return
        channel_id = msg['channel']
        msg_text = msg['text']
        parsed = does_msg_contain_prefix("!help", msg_text)
        if parsed:
            yield from self.send_help_message(channel_id)

    @asyncio.coroutine
    def send_help_message(self, channel_id):
        self.sc.rtm_send_message(channel_id, self.get_help_message())



import unittest
from charlesbot.util.parse import does_msg_contain_prefix


class TestDoesMsgContainPrefix(unittest.TestCase):

    def test_prefix_one(self):
        msg = "!HELP"
        retval = does_msg_contain_prefix("!help", msg)
        self.assertTrue(retval)

    def test_prefix_two(self):
        msg = "!HelP"
        retval = does_msg_contain_prefix("!help", msg)
        self.assertTrue(retval)

    def test_prefix_three(self):
        msg = "!help"
        retval = does_msg_contain_prefix("!help", msg)
        self.assertTrue(retval)

    def test_prefix_four(self):
        msg = "!helps"
        retval = does_msg_contain_prefix("!help", msg)
        self.assertFalse(retval)

    def test_prefix_five(self):
        msg = "!helps "
        retval = does_msg_contain_prefix("!help", msg)
        self.assertFalse(retval)

    def test_prefix_six(self):
        msg = "!helps asdfsdf"
        retval = does_msg_contain_prefix("!help", msg)
        self.assertFalse(retval)

    def test_prefix_seven(self):
        msg = "!help asdfsadfa"
        retval = does_msg_contain_prefix("!help", msg)
        self.assertTrue(retval)

    def test_prefix_eight(self):
        msg = "!help  asdfsadfa"
        retval = does_msg_contain_prefix("!help", msg)
        self.assertTrue(retval)

    def test_prefix_nine(self):
        msg = "!help  "
        retval = does_msg_contain_prefix("!help", msg)
        self.assertTrue(retval)

    def test_prefix_ten(self):
        msg = " !help  "
        retval = does_msg_contain_prefix("!help", msg)
        self.assertTrue(retval)

    def test_prefix_eleven(self):
        msg = "  !help  "
        retval = does_msg_contain_prefix("!help", msg)
        self.assertTrue(retval)

    def test_prefix_twelve(self):
        msg = "!helpsasdfsdf"
        retval = does_msg_contain_prefix("!help", msg)
        self.assertFalse(retval)

    def test_prefix_thirteen(self):
        msg = "  !help lllllllllllllllll  asdfsadf"
        retval = does_msg_contain_prefix("!help", msg)
        self.assertTrue(retval)

    def test_prefix_fourteen(self):
        msg = "  !help                                    "
        retval = does_msg_contain_prefix("!help", msg)
        self.assertTrue(retval)



import asynctest
from asynctest.mock import call
from asynctest.mock import patch
from asynctest.mock import MagicMock


class TestMisc(asynctest.TestCase):

    @asynctest.ignore_loop
    def test_get_plugin_name(self):
        with patch('charlesbot.plugins.broadcast_message.BroadcastMessage.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.broadcast_message import BroadcastMessage
            mock_init.return_value = None
            slack_client = MagicMock()
            t_bm = BroadcastMessage(slack_client)
            t_bm.sc = slack_client
            t_bm._plugin_name = "fancy plugin"
            self.assertEqual(t_bm.get_plugin_name(), "fancy plugin")

    def test_seed_robot_info(self):
        with patch('charlesbot.plugins.broadcast_message.BroadcastMessage.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.broadcast_message import BroadcastMessage
            mock_init.return_value = None
            slack_client = MagicMock()
            with patch('charlesbot.plugins.broadcast_message.slack_rtm_api_call') as mock_slack:  # NOQA
                with patch('charlesbot.plugins.broadcast_message.get_robot_info') as mock_robot:  # NOQA
                    mock_robot.return_value = {"U1234": "robotname"}
                    t_bm = BroadcastMessage(slack_client)
                    t_bm.sc = slack_client
                    t_bm.log = MagicMock()
                    yield from t_bm.seed_robot_info()
                    self.assertEqual(
                        t_bm.log.mock_calls,
                        [call('Robot username: robotname')]
                    )



import charlesbot.plugins as m


def get_plugin_class(class_str, slack_client):
    """Convenience function to convert a string into a class name"""
    return getattr(m, class_str)(slack_client)


def initialize_plugins(slack_client, config_plugin_list_str):
    return_list = []
    if not config_plugin_list_str:
        return return_list
    p_list = config_plugin_list_str.split(',')
    for x in p_list:
        Pluginz = get_plugin_class(x, slack_client)
        return_list.append(Pluginz)
    return return_list



import asynctest
from asynctest.mock import call
from asynctest.mock import patch
from asynctest.mock import MagicMock


class TestPrintRoomMembership(asynctest.TestCase):

    @asynctest.ignore_loop
    def test_print_room_membership_empty_dict(self):
        with patch('charlesbot.plugins.broadcast_message.BroadcastMessage.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.broadcast_message import BroadcastMessage
            mock_init.return_value = None
            slack_client = MagicMock()
            t_bm = BroadcastMessage(slack_client)
            t_bm.sc = slack_client
            t_bm.log = MagicMock()
            t_bm.room_membership = {}
            t_bm.print_room_membership()
            self.assertEqual(t_bm.log.mock_calls, [call('Currently in: ')])

    @asynctest.ignore_loop
    def test_print_room_membership_single(self):
        with patch('charlesbot.plugins.broadcast_message.BroadcastMessage.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.broadcast_message import BroadcastMessage
            mock_init.return_value = None
            slack_client = MagicMock()
            t_bm = BroadcastMessage(slack_client)
            t_bm.sc = slack_client
            t_bm.log = MagicMock()
            t_bm.room_membership = {"1234": "fun"}
            t_bm.print_room_membership()
            self.assertEqual(t_bm.log.mock_calls, [call('Currently in: fun')])

    @asynctest.ignore_loop
    def test_print_room_membership_multiple(self):
        with patch('charlesbot.plugins.broadcast_message.BroadcastMessage.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.broadcast_message import BroadcastMessage
            mock_init.return_value = None
            slack_client = MagicMock()
            t_bm = BroadcastMessage(slack_client)
            t_bm.sc = slack_client
            t_bm.log = MagicMock()
            t_bm.room_membership = {"1234": "fun", "4567": "funner"}
            t_bm.print_room_membership()
            self.assertEqual(
                t_bm.log.mock_calls,
                [call('Currently in: fun, funner')]
            )



import re
import logging
log = logging.getLogger(__name__)


def parse_msg_with_prefix(prefix, msg):
    regex_str = "^\s*%s[ :](.*)" % prefix
    p = re.compile(regex_str, re.IGNORECASE)
    m = p.match(msg)
    if m:
        return m.group(1).strip()
    else:
        return None


def filter_message_types(msg, types, fields):
    if 'type' not in msg.keys():
        return False
    if msg['type'] in types:
        if set(fields).issubset(msg.keys()):
            return True
    return False


def does_msg_contain_prefix(prefix, msg):
    regex_str = "^\s*%s(\s+\S+.*|\s+)?$" % prefix
    p = re.compile(regex_str, re.IGNORECASE)
    m = p.match(msg)
    if m:
        return True
    else:
        return False



import json
import logging
import asyncio
log = logging.getLogger(__name__)


@asyncio.coroutine
def slack_rtm_api_call(slack_client, api_endpoint, **kwargs):
    val = slack_client.api_call(api_endpoint, **kwargs)
    json_str = json.loads(val.decode('utf-8'))
    if not json_str['ok']:
        log.error(
            "Error fetching %s - response: %s",
            api_endpoint,
            str(json_str['ok'])
        )
        log.error(json_str)
        return json.dumps("[]")
    return json.dumps(json_str)


def parse_user_info(users_info):
    user = {}
    json_resp = json.loads(users_info)
    user.update({"id": json_resp['user']['id']})
    user.update({"username": json_resp['user']['name']})
    user.update({"real_name": json_resp['user']['profile']['real_name']})
    user.update({"thumb_24": json_resp['user']['profile']['image_24']})
    return user


def get_robot_info(auth_test):
    json_str = json.loads(auth_test)
    robot_name = json_str["user"]
    robot_id = json_str["user_id"]
    return {robot_id: robot_name}


def get_robot_channel_membership(channel_list):
    json_list = json.loads(channel_list)
    membership = {}
    for channel in json_list['channels']:
        if channel['is_member'] and not channel['is_general']:
            channel_id = channel['id']
            channel_name = channel['name']
            membership[channel_id] = channel_name
    return membership


def get_robot_group_membership(group_list):
    json_list = json.loads(group_list)
    membership = {}

    if 'groups' not in json_list:
        return membership

    for group in json_list['groups']:
        group_id = group['id']
        group_name = group['name']
        membership[group_id] = group_name
    return membership



import asynctest
from asynctest.mock import call
from asynctest.mock import patch
from asynctest.mock import MagicMock


class TestRemoveFromRoom(asynctest.TestCase):

    def test_remove_not_in_any_rooms(self):
        with patch('charlesbot.plugins.broadcast_message.BroadcastMessage.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.broadcast_message import BroadcastMessage
            mock_init.return_value = None
            slack_client = asynctest.MagicMock()
            with patch('charlesbot.plugins.broadcast_message.filter_message_types') as mock_filter:  # NOQA
                mock_filter.return_value = True
                t_bm = BroadcastMessage(slack_client)
                t_bm.sc = slack_client
                t_bm.log = MagicMock()
                t_bm.print_room_membership = MagicMock()
                t_bm.room_membership = {}
                msg = {
                    "type": "channel_left",
                    "channel": "C024BE91L"
                }
                yield from t_bm.remove_from_room(msg)
                self.assertEqual(t_bm.log.mock_calls, [])
                self.assertEqual(t_bm.room_membership, {})

    def test_remove_bogus_room(self):
        with patch('charlesbot.plugins.broadcast_message.BroadcastMessage.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.broadcast_message import BroadcastMessage
            mock_init.return_value = None
            slack_client = asynctest.MagicMock()
            with patch('charlesbot.plugins.broadcast_message.filter_message_types') as mock_filter:  # NOQA
                mock_filter.return_value = True
                t_bm = BroadcastMessage(slack_client)
                t_bm.sc = slack_client
                t_bm.log = MagicMock()
                t_bm.print_room_membership = MagicMock()
                t_bm.room_membership = {"5678": "funner"}
                msg = {
                    "type": "channel_left",
                    "channel": "1234"
                }
                yield from t_bm.remove_from_room(msg)
                self.assertEqual(t_bm.log.mock_calls, [])
                self.assertEqual(t_bm.room_membership, {"5678": "funner"})

    def test_remove_in_one_room(self):
        with patch('charlesbot.plugins.broadcast_message.BroadcastMessage.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.broadcast_message import BroadcastMessage
            mock_init.return_value = None
            slack_client = asynctest.MagicMock()
            with patch('charlesbot.plugins.broadcast_message.filter_message_types') as mock_filter:  # NOQA
                mock_filter.return_value = True
                t_bm = BroadcastMessage(slack_client)
                t_bm.sc = slack_client
                t_bm.log = MagicMock()
                t_bm.print_room_membership = MagicMock()
                t_bm.room_membership = {"1234": "funner"}
                msg = {
                    "type": "channel_left",
                    "channel": "1234"
                }
                yield from t_bm.remove_from_room(msg)
                self.assertEqual(
                    t_bm.log.mock_calls,
                    [call('I have left funner')]
                )
                self.assertEqual(t_bm.room_membership, {})

    def test_remove_in_two_rooms(self):
        with patch('charlesbot.plugins.broadcast_message.BroadcastMessage.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.broadcast_message import BroadcastMessage
            mock_init.return_value = None
            slack_client = asynctest.MagicMock()
            with patch('charlesbot.plugins.broadcast_message.filter_message_types') as mock_filter:  # NOQA
                mock_filter.return_value = True
                t_bm = BroadcastMessage(slack_client)
                t_bm.sc = slack_client
                t_bm.log = MagicMock()
                t_bm.print_room_membership = MagicMock()
                t_bm.room_membership = {"1234": "funner", "4567": "moar_fun"}
                msg = {
                    "type": "channel_left",
                    "channel": "4567"
                }
                yield from t_bm.remove_from_room(msg)
                self.assertEqual(
                    t_bm.log.mock_calls,
                    [call('I have left moar_fun')]
                )
                self.assertEqual(t_bm.room_membership, {"1234": "funner"})



import unittest
from unittest.mock import patch
from slackclient import SlackClient


class TestRobotUtils(unittest.TestCase):

    def mock_get_plugin_class(self, input_string, slack_client):
        return input_string

    def test_initialize_plugins_single(self):
        with patch('charlesbot.util.plugins.get_plugin_class') as mock1:
            from charlesbot.util.plugins import initialize_plugins
            mock1.side_effect = self.mock_get_plugin_class
            input_list = "oneplugin"
            expected_output = ["oneplugin"]
            fake_client = SlackClient("faketoken")
            self.assertCountEqual(
                initialize_plugins(fake_client, input_list),
                expected_output
            )

    def test_initialize_plugins_multiple(self):
        with patch('charlesbot.util.plugins.get_plugin_class') as mock1:
            from charlesbot.util.plugins import initialize_plugins
            mock1.side_effect = self.mock_get_plugin_class
            input_list = "one,two"
            expected_output = ['one', 'two']
            fake_client = SlackClient("faketoken")
            self.assertCountEqual(
                initialize_plugins(fake_client, input_list),
                expected_output
            )

    def test_initialize_plugins_empty(self):
        with patch('charlesbot.util.plugins.get_plugin_class') as mock1:
            from charlesbot.util.plugins import initialize_plugins
            mock1.side_effect = self.mock_get_plugin_class
            input_list = ""
            expected_output = []
            fake_client = SlackClient("faketoken")
            self.assertEqual(
                initialize_plugins(fake_client, input_list),
                expected_output
            )



import unittest
import os
from unittest.mock import patch
from charlesbot.util.config import get_config_file_name


class TestConfig(unittest.TestCase):

    @patch.dict('os.environ', {'CHARLESBOT_SETTINGS_FILE': 'filename'})
    def test_config_file_name(self):
        self.assertEqual(get_config_file_name(), 'filename')

    @patch.dict('os.environ', {'CHARLESBOT_SETTINGS_FILE': ''})
    def test_config_file_name_empty(self):
        self.assertEqual(get_config_file_name(), './development.ini')

    @patch.dict('os.environ', {'CHARLESBOT_SETTINGS_FILE': 'filename'})
    def test_config_file_name_not_present(self):
        del os.environ['CHARLESBOT_SETTINGS_FILE']
        self.assertEqual(get_config_file_name(), './development.ini')



from charlesbot.util.slack import slack_rtm_api_call
from charlesbot.util.slack import get_robot_channel_membership
from charlesbot.util.slack import get_robot_group_membership
from charlesbot.util.slack import get_robot_info
from charlesbot.util.slack import parse_user_info
from charlesbot.util.parse import parse_msg_with_prefix
from charlesbot.util.parse import filter_message_types
import logging
import asyncio
import json


class BroadcastMessage(object):

    def __init__(self, slack_client):
        self.is_running = True
        self._plugin_name = "Broadcast Message"
        self.log = logging.getLogger(__name__)
        self.sc = slack_client
        self.room_membership = {}
        self.robot_info = {}
        self.log.info("Initializing the Broadcast Message plugin")
        loop = asyncio.get_event_loop()
        loop.create_task(self.seed_channel_membership())
        loop.create_task(self.seed_group_membership())
        loop.create_task(self.seed_robot_info())
        self.q = asyncio.Queue()
        loop.create_task(self.consume())

    def get_plugin_name(self):
        return self._plugin_name

    @asyncio.coroutine
    def seed_robot_info(self):
        result = yield from slack_rtm_api_call(self.sc, 'auth.test')
        self.robot_info = get_robot_info(result)
        name = next(iter(self.robot_info.values()))
        self.log.info("Robot username: %s" % name)

    def print_room_membership(self):
        self.log.info("Currently in: %s"
                      % ", ".join(sorted(self.room_membership.values())))

    @asyncio.coroutine
    def seed_channel_membership(self):
        result = yield from slack_rtm_api_call(self.sc,
                                               'channels.list',
                                               exclude_archived=1)
        channels = get_robot_channel_membership(result)
        self.room_membership.update(channels)
        self.print_room_membership()

    @asyncio.coroutine
    def seed_group_membership(self):
        result = yield from slack_rtm_api_call(self.sc,
                                               'groups.list',
                                               exclude_archived=1)
        groups = get_robot_group_membership(result)
        self.room_membership.update(groups)
        self.print_room_membership()

    @asyncio.coroutine
    def process_message(self, messages):
        for msg in messages:
            tasks = [
                self.add_to_room(msg),
                self.remove_from_room(msg),
                self.parse_wall_message(msg)
            ]
            yield from asyncio.gather(*tasks)

    @asyncio.coroutine
    def add_to_room(self, msg):
        types = ['group_joined', 'channel_joined']
        fields = ['channel']
        if not filter_message_types(msg, types, fields):
            return
        room_name = msg['channel']['name']
        room_id = msg['channel']['id']
        self.room_membership.update({room_id: room_name})
        self.log.info("I was invited to join %s" % room_name)
        self.print_room_membership()

    @asyncio.coroutine
    def remove_from_room(self, msg):
        types = ['group_left', 'channel_left']
        fields = ['channel']
        if not filter_message_types(msg, types, fields):
            return
        room_id = msg['channel']

        room_name = ""
        if room_id in self.room_membership:
            room_name = self.room_membership[room_id]

        self.room_membership.pop(room_id, None)

        if room_name:
            self.log.info("I have left %s" % room_name)

        self.print_room_membership()

    @asyncio.coroutine
    def parse_wall_message(self, msg):
        types = ['message']
        fields = ['channel', 'user', 'text']
        if not filter_message_types(msg, types, fields):
            return
        sent_by_id = msg['user']
        msg_text = msg['text']
        parsed = parse_msg_with_prefix("!wall", msg_text)
        if parsed:
            user = yield from self.get_user_info(sent_by_id)
            yield from self.send_broadcast_message(parsed, user)

    @asyncio.coroutine
    def get_user_info(self, user_id):
        result = yield from slack_rtm_api_call(
            self.sc,
            'users.info',
            user=user_id
        )
        return parse_user_info(result)

    @asyncio.coroutine
    def send_broadcast_message(self, msg, user):
        wall = "Broadcast message from %s - %s" % (user['real_name'], msg)

        attachment = [
            {
                "fallback": wall,
                "author_name": user['real_name'],
                "author_icon": user['thumb_24'],
                "text": msg,
            }
        ]

        for key in self.room_membership.keys():
            yield from slack_rtm_api_call(
                self.sc,
                'chat.postMessage',
                channel=key,
                attachments=json.dumps(attachment),
                as_user=False,
                username="Broadcast Message",
                icon_url="https://s3-us-west-2.amazonaws.com/slack-files2/bot_icons/2015-07-26/8217958308_48.png"  # NOQA
            )

    @asyncio.coroutine
    def consume(self):
        while self.is_running:
            value = yield from self.q.get()
            loop = asyncio.get_event_loop()
            loop.create_task(self.process_message(value))



import asynctest
from asynctest.mock import patch
from asynctest.mock import MagicMock


class TestMisc(asynctest.TestCase):

    @asynctest.ignore_loop
    def test_get_plugin_name(self):
        with patch('charlesbot.plugins.help_plugin.Help.__init__') as mock_init:  # NOQA
            from charlesbot.plugins.help_plugin import Help
            mock_init.return_value = None
            slack_client = MagicMock()
            t_bm = Help(slack_client)
            t_bm.sc = slack_client
            t_bm._plugin_name = "fancy plugin"
            self.assertEqual(t_bm.get_plugin_name(), "fancy plugin")



import unittest
from charlesbot.util.parse import parse_msg_with_prefix
from charlesbot.util.parse import filter_message_types


class TestMessageParser(unittest.TestCase):

    def test_prefix_uppercase(self):
        msg = "!ALL hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_mixed(self):
        msg = "!AlL hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_colon(self):
        msg = "!all: hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_colon_two(self):
        msg = "!all:hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_space(self):
        msg = "!all hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_whitespace(self):
        msg = "!all  hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_leading_whitespace(self):
        msg = " !all  hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_leading_whitespace_two(self):
        msg = "  !all  hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual("hi, there!", retval)

    def test_prefix_invalid_one(self):
        msg = "s  !all  hi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual(None, retval)

    def test_prefix_invalid_two(self):
        msg = "!allhi, there!"
        retval = parse_msg_with_prefix("!all", msg)
        self.assertEqual(None, retval)

    def test_filter_okay(self):
        msg = {"type": "group_joined", "channel": "fun"}
        types = ['group_joined', 'channel_joined']
        fields = ['channel']
        self.assertEqual(filter_message_types(msg, types, fields), True)

    def test_filter_type_not_present(self):
        msg = {"types": "group_joined", "channel": "fun"}
        types = ['group_joined', 'channel_joined']
        fields = ['channel']
        self.assertEqual(filter_message_types(msg, types, fields), False)

    def test_filter_type_not_in_types(self):
        msg = {"type": "group_joined", "channel": "fun"}
        types = ['group_joining', 'channel_joined']
        fields = ['channel']
        self.assertEqual(filter_message_types(msg, types, fields), False)

    def test_filter_field_not_present(self):
        msg = {"type": "group_joined", "channel": "fun"}
        types = ['group_joined', 'channel_joined']
        fields = ['channels']
        self.assertEqual(filter_message_types(msg, types, fields), False)

    def test_filter_too_many_fields(self):
        msg = {"type": "group_joined", "channel": "fun"}
        types = ['group_joined', 'channel_joined']
        fields = ['channel', 'type', 'channels']
        self.assertEqual(filter_message_types(msg, types, fields), False)

    def test_filter_exact_amount_of_fields(self):
        msg = {"type": "group_joined", "channel": "fun"}
        types = ['group_joined', 'channel_joined']
        fields = ['channel', 'type']
        self.assertEqual(filter_message_types(msg, types, fields), True)

    def test_filter_no_fields(self):
        msg = {"type": "group_joined", "channel": "fun"}
        types = ['group_joined', 'channel_joined']
        fields = []
        self.assertEqual(filter_message_types(msg, types, fields), True)



import asynctest
import json
from unittest.mock import MagicMock
from charlesbot.util.slack import slack_rtm_api_call
from slackclient import SlackClient
from tests import load_fixture


class TestSlackRtmApiCall(asynctest.TestCase):

    def setUp(self):
        self.channel_list_one = load_fixture('channel_list_one.json')
        self.channel_list_two = load_fixture('channel_list_two.json')
        self.channel_list_three = load_fixture('channel_list_three.json')
        self.channel_list_four = load_fixture('channel_list_four.json')

    def test_slack_rtm_api_call_ok(self):
        channel_list = self.channel_list_one.encode('utf8')
        fake_client = SlackClient("faketoken")
        fake_client.api_call = MagicMock(return_value=channel_list)
        val = yield from slack_rtm_api_call(fake_client, "fake_endpoint")
        self.assertEqual(
            json.loads(val),
            json.loads(channel_list.decode('utf-8'))
        )

    def test_slack_rtm_api_call_not_ok(self):
        channel_list = self.channel_list_two.encode('utf8')
        fake_client = SlackClient("faketoken")
        fake_client.api_call = MagicMock(return_value=channel_list)
        val = yield from slack_rtm_api_call(fake_client, "fake_endpoint")
        self.assertEqual(json.loads(val), '[]')

    def test_slack_rtm_api_call_not_encoded_utf8(self):
        channel_list = self.channel_list_three
        fake_client = SlackClient("faketoken")
        fake_client.api_call = MagicMock(return_value=channel_list)
        with self.assertRaises(AttributeError):
            yield from slack_rtm_api_call(fake_client, "fake_endpoint")

    def test_slack_rtm_api_call_malformed_json(self):
        channel_list = self.channel_list_four.encode('utf8')
        fake_client = SlackClient("faketoken")
        fake_client.api_call = MagicMock(return_value=channel_list)
        with self.assertRaises(ValueError):
            yield from slack_rtm_api_call(fake_client, "fake_endpoint")



import unittest
from charlesbot.util.slack import parse_user_info
import json


class TestParseUserInfo(unittest.TestCase):

    def test_user_info_ok(self):
        users_info = {
            "ok": True,
            "user": {
                "id": "U023BECGF",
                "name": "bobby",
                "deleted": False,
                "color": "9f69e7",
                "profile": {
                    "first_name": "Bobby",
                    "last_name": "Tables",
                    "real_name": "Bobby Tables",
                    "email": "bobby@slack.com",
                    "skype": "my-skype-name",
                    "phone": "+1 (123) 456 7890",
                    "image_24": "https://www.tables.com",
                    "image_32": "https:\/\/...",
                    "image_48": "https:\/\/...",
                    "image_72": "https:\/\/...",
                    "image_192": "https:\/\/..."
                },
                "is_admin": True,
                "is_owner": True,
                "has_2fa": True,
                "has_files": True
            }
        }
        retval = parse_user_info(json.dumps(users_info))
        expected = {
            "id": "U023BECGF",
            "username": "bobby",
            "real_name": "Bobby Tables",
            "thumb_24": "https://www.tables.com"
        }
        self.assertEqual(retval, expected)

    def test_user_info_missing(self):
        users_info = {
            "ok": True,
            "user": {
                "name": "bobby",
                "deleted": False,
                "color": "9f69e7",
                "profile": {
                    "first_name": "Bobby",
                    "last_name": "Tables",
                    "email": "bobby@slack.com",
                    "skype": "my-skype-name",
                    "phone": "+1 (123) 456 7890",
                    "image_32": "https:\/\/...",
                    "image_48": "https:\/\/...",
                    "image_72": "https:\/\/...",
                    "image_192": "https:\/\/..."
                },
                "is_admin": True,
                "is_owner": True,
                "has_2fa": True,
                "has_files": True
            }
        }
        with self.assertRaises(KeyError):
            parse_user_info(json.dumps(users_info))



from charlesbot.util.config import get_config_file_name
from charlesbot.util.config import read_config
from charlesbot.util.plugins import initialize_plugins
from slackclient import SlackClient
import logging
import asyncio
import sys
import signal
import traceback
import functools


class Robot(object):

    def __init__(self):
        self.config = read_config(get_config_file_name())
        self.log = logging.getLogger(__name__)
        self.token = self.config.get('main', 'slackbot_token')
        self.enabled_plugins = self.config.get('main', 'enabled_plugins')
        self.sc = None
        self.is_running = True

    def connect(self):
        """Convenience method that creates Server instance"""
        self.sc = SlackClient(self.token)
        return self.sc.rtm_connect()

    @asyncio.coroutine
    def produce(self):
        while self.is_running:
            yield from self.route_message_to_plugin()

    @asyncio.coroutine
    def route_message_to_plugin(self):
        try:
            msg = self.sc.rtm_read()
            for plug in self.plugin_list:
                yield from self.queue_message(msg, plug)
        except BrokenPipeError as b:
            self.log.error("Error reading from slack socket: %s" % b)
            self.log.debug(traceback.format_exc())
        yield from asyncio.sleep(0.5)

    @asyncio.coroutine
    def queue_message(self, message, plugin):
        if message:
            self.log.debug(
                "Routing message %s to plugin %s" % (message, plugin)
            )
            yield from plugin.q.put(message)

    def exit_cleanly(self):
        loop = asyncio.get_event_loop()
        self.log.info("Shutting down charlesbot")
        self.is_running = False

        for plug in self.plugin_list:
            self.log.info("Shutting down plugin: %s" % plug.get_plugin_name())
            plug.is_running = False

        pending = asyncio.Task.all_tasks()
        asyncio.gather(*pending).cancel()
        loop.stop()

    def start(self):
        if not self.connect():
            self.log.error("Error conecting to Slack - possible token issue?")
            sys.exit(1)
        loop = asyncio.get_event_loop()
        self.plugin_list = initialize_plugins(self.sc, self.enabled_plugins)
        loop.create_task(self.produce())
        loop.add_signal_handler(
            signal.SIGINT,
            functools.partial(self.exit_cleanly)
        )
        try:
            loop.run_forever()
        finally:
            loop.close()



import os
from configparser import ConfigParser


def get_config_file_name():
    if 'CHARLESBOT_SETTINGS_FILE' in os.environ:
        if os.environ['CHARLESBOT_SETTINGS_FILE']:
            return os.environ['CHARLESBOT_SETTINGS_FILE']
    return './development.ini'


def read_config(config_file):
    config = ConfigParser(allow_no_value=False)
    config.read(config_file)
    return config
